﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

using LogSig.Crypto;
using LogSig.DM;
using LogSig.DM.LsGen;
using LogSig.Pkc;
using LogSig.Skc;

namespace LogSig.AppModules
{
	public sealed class AppModPgm : AppMod
	{
		private const string StrPreReq = "pgm";

		public AppModPgm()
		{
			AddCommand("c", CreateSecretKey);
		}

		private static void CreateSecretKey(object sender, AppModEventArgs e)
		{
			if(e.Args[StrPreReq] == null) return;

			Program.GroupFactories.InvokeGroupMethod(e, typeof(AmpCreateSecretKey), e);
		}

		internal static class AmpCreateSecretKey
		{
			public static void Invoke<T>(IDmGroup<T> grp, AppModEventArgs e)
				where T : IDmGroupElement<T>
			{
				string strCmdBaseName = e.Args["base"];
				string strBaseName = (strCmdBaseName ?? "Generated");

				ulong uRounds;
				if(!ulong.TryParse(e.Args["r"], out uRounds)) uRounds = 5000;

				string strPassword = e.Args["pw"];
				if(string.IsNullOrEmpty(strPassword))
				{
					Console.WriteLine("A password must be passed using the '-pw' option.");
					return;
				}

				byte[] pbKey32 = new byte[32];
				byte[] pbCtr16 = new byte[16];
				List<int> lFac;
				SkcKey sk = new SkcKey();

				string strTrfIORels = string.Empty;
				Program.CryptoRandom.Generate(pbKey32, 0, pbKey32.Length);
				Program.CryptoRandom.Generate(pbCtr16, 0, pbCtr16.Length);
				CryptoRandomGen rand = new AesCryptoRandomGen(pbKey32, pbCtr16);
				var ls = LsGen<T>.Generate(grp, uRounds, rand, default(T),
					ref strTrfIORels, out lFac);
				sk.Alpha = PkcPrivateKey.CreatePrivate(grp, ls, pbKey32,
					pbCtr16, uRounds, strTrfIORels, strPassword);

				strTrfIORels = string.Empty;
				Program.CryptoRandom.Generate(pbKey32, 0, pbKey32.Length);
				Program.CryptoRandom.Generate(pbCtr16, 0, pbCtr16.Length);
				rand = new AesCryptoRandomGen(pbKey32, pbCtr16);
				ls = LsGen<T>.Generate(grp, uRounds, rand, default(T),
					ref strTrfIORels, out lFac);
				sk.Beta = PkcPrivateKey.CreatePrivate(grp, ls, pbKey32,
					pbCtr16, uRounds, strTrfIORels, strPassword);

				using(FileStream fs = new FileStream(strBaseName + ".key",
					FileMode.Create, FileAccess.Write, FileShare.None))
				{
					PkcKeyFile.Save(fs, sk);
				}
			}
		}
	}
}
