﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Text;
using System.Numerics;

namespace LogSig.DM.DirectProductGroup
{
	public sealed class DmDirectProductGroupElement<U, V> :
		IDmGroupElement<DmDirectProductGroupElement<U, V>>
		where U : IDmGroupElement<U>
		where V : IDmGroupElement<V>
	{
		private int? m_hash = null;

		private readonly U m_u;
		public U Value0
		{
			get { return m_u; }
			// set { m_u = value; } // Immutable
		}

		private readonly V m_v;
		public V Value1
		{
			get { return m_v; }
			// set { m_v = value; } // Immutable
		}

		public DmDirectProductGroupElement(U u, V v)
		{
			m_u = u;
			m_v = v;
		}

		public DmDirectProductGroupElement<U, V> Clone()
		{
			return (DmDirectProductGroupElement<U, V>)MemberwiseClone();
		}

		public override string ToString()
		{
			return (m_u.ToString() + "x" + m_v.ToString());
		}

		public static DmDirectProductGroupElement<U, V> Parse(string str,
			IDmGroup<U> grpU, IDmGroup<V> grpV)
		{
			if(str == null) throw new ArgumentNullException("str");

			str = str.Trim().ToLower();

			int iSplit = str.IndexOf('x');
			if(iSplit < 0) throw new FormatException();

			string strU = str.Substring(0, iSplit);
			string strV = str.Substring(iSplit + 1);

			return new DmDirectProductGroupElement<U, V>(grpU.ParseElement(strU),
				grpV.ParseElement(strV));
		}

		public override int GetHashCode()
		{
			if(m_hash.HasValue) return m_hash.Value;

			int h = m_u.GetHashCode();
			h += (h << 10);
			h ^= (h >> 6);
			h += m_v.GetHashCode();

			m_hash = h;
			return h;
		}

		public override bool Equals(object obj)
		{
			var other = (obj as DmDirectProductGroupElement<U, V>);
			return Equals(other);
		}

		public bool Equals(DmDirectProductGroupElement<U, V> other)
		{
			if(other == null) return false;
			return (m_u.Equals(other.m_u) && m_v.Equals(other.m_v));
		}

		public int CompareTo(DmDirectProductGroupElement<U, V> other)
		{
			if(other == null) return 1;

			int cmpU = m_u.CompareTo(other.m_u);
			if(cmpU != 0) return cmpU;

			return m_v.CompareTo(other.m_v);
		}
	}
}
