﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Diagnostics;

namespace LogSig.DM.DirectProductGroup
{
	public sealed class DmDirectProductGroupFactory : IDmGroupFactory
	{
		public object CreateInstance(string strGroup)
		{
			int iSplit = strGroup.IndexOfAny(new char[] { 'x', 'X' });
			if(iSplit < 0) return null;

			string strU = strGroup.Substring(0, iSplit);
			string strV = strGroup.Substring(iSplit + 1);

			object grpU = Program.GroupFactories.ConstructGroup(strU);
			object grpV = Program.GroupFactories.ConstructGroup(strV);
			if((grpU == null) || (grpV == null)) return null;

			Type tElemU = DmGroupFactoryPool.GetGroupElementType(grpU.GetType());
			Type tElemV = DmGroupFactoryPool.GetGroupElementType(grpV.GetType());

			Assembly asm = Assembly.GetExecutingAssembly();
			foreach(Type t in asm.GetTypes())
			{
				if(t.Name.StartsWith(@"DmDirectProductGroup`"))
				{
					Type tProductGroup = t.MakeGenericType(tElemU, tElemV);
					var vCtor = tProductGroup.GetConstructors();
					if((vCtor == null) || (vCtor.Length != 1))
						throw new InvalidOperationException();

					return vCtor[0].Invoke(new object[] { grpU, grpV });
				}
			}

			Debug.Assert(false);
			throw new InvalidOperationException();
		}
	}
}
