﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using LogSig.Crypto;
using LogSig.Interfaces;

namespace LogSig.DM
{
	// The following is deprecated; use LsGen instead

	/*
	public sealed class DmLogSigGenOptions
	{
		private bool m_bFuseInMiddle = false;
		public bool FuseInMiddle
		{
			get { return m_bFuseInMiddle; }
			set { m_bFuseInMiddle = value; }
		}
	}

	public static class DmLogSigGen
	{
		[Obsolete("Use LsGen instead.")]
		public static DmBlockSeq<T> Generate<T>(IDmGroup<T> grp, ulong uRounds,
			CryptoRandomGen rand, out T tLeft, out T tRight,
			int[] vFactorization = null, DmLogSigGenOptions opt = null)
			where T : IDmGroupElement<T>
		{
			if(grp == null) throw new ArgumentNullException("grp");
			if(rand == null) throw new ArgumentNullException("rand");

			DmBlockSeq<T> ls = grp.CreateDefaultSig();
			if((vFactorization != null) && (ls.Blocks.Count != vFactorization.Length))
				throw new ArgumentException("vFactorization");

			tLeft = grp.Identity;
			tRight = grp.Identity;

			for(ulong r = 0; r < uRounds; ++r)
			{
				SubstituteBlocks(ls, 2, grp, rand, vFactorization);
				// SubstituteBlocks(ls, 3, grp, rand, vFactorization);
				FrontTranslate(ls, grp, rand, ref tLeft);
				// Sandwich(ls, grp, rand);
				ShuffleBlocks(ls, rand, vFactorization);
				Normalize(ref ls, grp, ref tRight);

				if((r == (uRounds / 2)) && (opt != null) && opt.FuseInMiddle)
				{
					// int nFusions = ls.Blocks.Count / 3;
					int nFusions = ls.Blocks.Count - 2;
					for(int i = 0; i < nFusions; ++i)
					{
						if(ls.Blocks.Count <= 2) continue;
						int x = rand.GenerateInt32(ls.Blocks.Count - 1);
						DmLogSigBlock<T> blkNew = DmLogSigBlock<T>.Fusion(
							ls.Blocks[x], ls.Blocks[x + 1], grp);
						ls.Blocks.RemoveAt(x + 1);
						ls.Blocks.RemoveAt(x);
						ls.Blocks.Insert(x, blkNew);
					}
				}
			}

			return ls;
		}

		private static void SubstituteBlocks<T>(DmBlockSeq<T> ls, int nBlocks,
			IDmGroup<T> grp, CryptoRandomGen rand, int[] vFactorization)
			where T : IDmGroupElement<T>
		{
			for(int x = 0; x < (ls.Blocks.Count - nBlocks + 1); ++x)
			{
				var lsSub = ls.GetSubSig(x, nBlocks);
				int nRejectChance = lsSub.Blocks.Sum(blk => blk.Elements.Count);

				var lsNewSub = lsSub.GetRandomEquivalentCanonical(grp, rand);
				if(lsSub != lsNewSub)
				{
					if(vFactorization != null)
					{
						List<int> vPre = vFactorization.Skip(x).Take(nBlocks).ToList();
						T tProd = lsSub.MultiplyElements(vPre, grp);
						int[] vPost = new int[nBlocks];
						if(!DmLogSigUtil.TrivialFactorize(lsNewSub, tProd, grp.Identity,
							0, grp, vPost)) { Debug.Assert(false); }
						Array.Copy(vPost, 0, vFactorization, x, nBlocks);
					}

					for(int xc = 0; xc < nBlocks; ++xc)
						ls.Blocks[x + xc] = lsNewSub.Blocks[xc];
				}
				else if(rand.GenerateInt32(nRejectChance) != 0) --x;
			}
		}

		private static void FrontTranslate<T>(DmBlockSeq<T> ls, IDmGroup<T> grp,
			CryptoRandomGen rand, ref T tLeft)
			where T : IDmGroupElement<T>
		{
			T elTrans = grp.GenerateRandomElement(rand);
			List<T> v = ls.Blocks[0].Elements;
			for(int y = 0; y < v.Count; ++y) v[y] = grp.Multiply(elTrans, v[y]);

			tLeft = grp.Multiply(elTrans, tLeft);
		}

		private static void Sandwich<T>(DmBlockSeq<T> ls, IDmGroup<T> grp,
			CryptoRandomGen rand)
			where T : IDmGroupElement<T>
		{
			for(int x = 0; x < (ls.Blocks.Count - 1); ++x)
			{
				List<T> vL = ls.Blocks[x].Elements;
				List<T> vR = ls.Blocks[x + 1].Elements;

				T elL = grp.GenerateRandomElement(rand);
				T elR = grp.Invert(elL);

				for(int y = 0; y < vL.Count; ++y) vL[y] = grp.Multiply(vL[y], elL);
				for(int y = 0; y < vR.Count; ++y) vR[y] = grp.Multiply(elR, vR[y]);
			}
		}

		private static void ShuffleBlocks<T>(DmBlockSeq<T> ls, CryptoRandomGen rand,
			int[] vFactorization)
			where T : IDmGroupElement<T>
		{
			for(int x = 0; x < ls.Blocks.Count; ++x)
			{
				List<T> l = ls.Blocks[x].Elements;
				if(l.Count == 0) { Debug.Assert(false); continue; }

				T tLocal = ((vFactorization != null) ? l[vFactorization[x]] : l[0]);

				DmLogSigUtil.ShuffleList(l, rand);

				if(vFactorization != null) vFactorization[x] = l.IndexOf(tLocal);
			}
		}

		private static void Normalize<T>(ref DmBlockSeq<T> ls, IDmGroup<T> grp,
			ref T tRight)
			where T : IDmGroupElement<T>
		{
			int cBlocks = ls.Blocks.Count;
			ls = DmLogSigUtil.GetLStarCanonical(ls, grp);

			Debug.Assert(ls.Blocks[ls.Blocks.Count - 1].Elements.Count == 1);
			T elTrans = ls.Blocks[ls.Blocks.Count - 1].Elements[0];
			tRight = grp.Multiply(tRight, grp.Invert(elTrans));

			if(ls.Blocks.Count != (cBlocks + 1)) { Debug.Assert(false); }
			else ls.Blocks.RemoveAt(ls.Blocks.Count - 1);
		}
	}
	*/
}
