﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Numerics;

using LogSig.Crypto;

namespace LogSig.DM
{
	public interface IDmGroup<T>
		where T : IDmGroupElement<T>
	{
		string Name { get; }

		// /// <summary>
		// /// Short name of the group (e.g. "Dih").
		// /// </summary>
		// string ShortName { get; }

		// /// <summary>
		// /// Alternative short name of the group (e.g. "Dn").
		// /// </summary>
		// string ShortNameAlt { get; }

		/// <summary>
		/// Order of the group (number of elements).
		/// </summary>
		BigInteger Order { get; }

		T Identity { get; }

		T ParseElement(string str);

		T Multiply(T a, T b);
		T Invert(T t);

		T GenerateRandomElement(CryptoRandomGen rand);
		T NextElement(T t);

		BigInteger ElementToBigInt(T t);
		T BigIntToElement(BigInteger biElem);

		DmBlockSeq<T> CreateDefaultSig();
		int[] FactorInDefaultSig(T t);
	}
}
