﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using LogSig.DM.LsGen.TransformationIORels;
using LogSig.Util;

namespace LogSig.DM.LsGen
{
	public abstract class TransformationIORel<T>
		where T : IDmGroupElement<T>
	{
		public abstract T GetInputReqForOutput(T tDesiredOutputElement,
			IDmGroup<T> grp);

		/// <summary>
		/// Deserialize a transformation I/O relation.
		/// If the string cannot be parsed, <c>null</c> is returned
		/// (no exception or assert).
		/// </summary>
		public abstract TransformationIORel<T> Parse(string str, IDmGroup<T> grp);

		public static T UndoList(T tDesiredOutputElement, IDmGroup<T> grp,
			List<TransformationIORel<T>> l)
		{
			T t = tDesiredOutputElement;
			for(int i = l.Count - 1; i >= 0; --i)
				t = l[i].GetInputReqForOutput(t, grp);
			return t;
		}

		public static void FuseList(List<TransformationIORel<T>> l, IDmGroup<T> grp)
		{
			for(int i = l.Count - 1; i >= 1; --i)
			{
				var lr1 = (l[i - 1] as LeftRightTranslationRel<T>);
				var lr2 = (l[i] as LeftRightTranslationRel<T>);
				if((lr1 != null) && (lr2 != null))
				{
					lr1.Left = grp.Multiply(lr2.Left, lr1.Left);
					lr1.Right = grp.Multiply(lr1.Right, lr2.Right);
					l.RemoveAt(i);
				}
			}
		}

		public static List<TransformationIORel<T>> GetAllRels()
		{
			var l = new List<TransformationIORel<T>>();

			l.Add(new LeftRightTranslationRel<T>());

			return l;
		}

		public static string ListToString(List<TransformationIORel<T>> l)
		{
			StringBuilder sb = new StringBuilder();
			foreach(TransformationIORel<T> t in l)
			{
				if(sb.Length > 0) sb.Append(',');
				sb.Append(t.ToString());
			}
			return sb.ToString();
		}

		public static List<TransformationIORel<T>> ParseList(string str,
			IDmGroup<T> grp)
		{
			var l = new List<TransformationIORel<T>>();
			var lAll = GetAllRels();

			List<string> vRels = StrUtil.SplitLevel0(str, new char[] { ',' });
			foreach(string strRel in vRels)
			{
				foreach(var ioRel in lAll)
				{
					TransformationIORel<T> rel = ioRel.Parse(strRel, grp);
					if(rel != null) { l.Add(rel); break; }
				}
			}

			return l;
		}
	}
}
