﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using LogSig.Util;

namespace LogSig.DM.LsGen.TransformationIORels
{
	public sealed class LeftRightTranslationRel<T> :
		TransformationIORel<T>
		where T : IDmGroupElement<T>
	{
		private T m_l;
		public T Left
		{
			get { return m_l; }
			set { m_l = value; }
		}

		private T m_r;
		public T Right
		{
			get { return m_r; }
			set { m_r = value; }
		}

		public LeftRightTranslationRel() { }

		public LeftRightTranslationRel(T tLeft, T tRight)
		{
			m_l = tLeft;
			m_r = tRight;
		}

		public override T GetInputReqForOutput(T tDesiredOutputElement,
			IDmGroup<T> grp)
		{
			return grp.Multiply(grp.Multiply(grp.Invert(m_l),
				tDesiredOutputElement), grp.Invert(m_r));
		}

		public override string ToString()
		{
			return ("(T:" + m_l.ToString() + "/" + m_r.ToString() + ")");
		}

		public override TransformationIORel<T> Parse(string str, IDmGroup<T> grp)
		{
			if((str == null) || !str.StartsWith("(T:") || !str.EndsWith(")"))
				return null;

			string strInner = str.Substring(3, str.Length - 4);
			List<string> vEl = StrUtil.SplitLevel0(strInner, new char[] { '/' });
			if((vEl == null) || (vEl.Count != 2)) { Debug.Assert(false); return null; }

			return new LeftRightTranslationRel<T>(grp.ParseElement(vEl[0]),
				grp.ParseElement(vEl[1]));
		}
	}
}
