﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using LogSig.Crypto;

namespace LogSig.DM.LsGen
{
	public sealed class TransformationParams<T>
		where T : IDmGroupElement<T>
	{
		public IDmGroup<T> Group { get; private set; }
		public CryptoRandomGen Random { get; private set; }
		public DmBlockSeq<T> LogSig { get; set; }
		public List<TransformationIORel<T>> TransformationIORels { get; private set; }
		public List<int> Factorization { get; set; }

		public TransformationParams(IDmGroup<T> grp, CryptoRandomGen rand,
			DmBlockSeq<T> ls, List<int> lFactorization)
		{
			if(grp == null) throw new ArgumentNullException("grp");
			if(rand == null) throw new ArgumentNullException("rand");
			if(ls == null) throw new ArgumentNullException("ls");
			// lFactorization may be null

			this.Group = grp;
			this.Random = rand;
			this.LogSig = ls;
			this.TransformationIORels = new List<TransformationIORel<T>>();
			this.Factorization = ((lFactorization != null) ? new List<int>(
				lFactorization) : null);
		}
	}
}
