﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using LogSig.DM.LsGen.GroupImplCaps;

namespace LogSig.DM.LsGen.Transformations
{
	public sealed class LsgtBlockShuffle<T> : LsGenTransformation<T>
		where T : IDmGroupElement<T>
	{
		public override bool SupportedForGroup(IDmGroup<T> grp)
		{
			return (grp is IFacPermShuffleTest<T>);
		}

		public override void Transform(TransformationParams<T> p)
		{
			IFacPermShuffleTest<T> fps = (p.Group as IFacPermShuffleTest<T>);
			if(fps == null) { Debug.Assert(false); return; }

			var vBlocks = p.LogSig.Blocks;
			int x = p.Random.GenerateInt32(vBlocks.Count);

			// Extend the selection as far to the left as possible
			// (i.e. as long as it is still guaranteed that every shuffle
			// of the blocks is factorization-permuting)
			int l = x;
			while(l > 0)
			{
				var lsSub = p.LogSig.GetSubSig(l - 1, x - l + 2);
				if(!fps.IsFacPermBlockSeq(lsSub.Blocks)) break;
				--l;
			}

			// Extend the selection to the right as above
			int r = x;
			while(r < (vBlocks.Count - 1))
			{
				var lsSub = p.LogSig.GetSubSig(l, r - l + 2);
				if(!fps.IsFacPermBlockSeq(lsSub.Blocks)) break;
				++r;
			}

			List<int> vPerm = new List<int>();
			for(int i = 0; i < (r - l + 1); ++i) vPerm.Add(i);
			DmLogSigUtil.ShuffleList(vPerm, p.Random);

			var vBlocksOrg = new List<DmLogSigBlock<T>>(vBlocks);
			List<int> vFacOrg = ((p.Factorization != null) ?
				new List<int>(p.Factorization) : null);
			for(int i = 0; i < vPerm.Count; ++i)
			{
				vBlocks[l + i] = vBlocksOrg[l + vPerm[i]];

				if(vFacOrg != null)
					p.Factorization[l + i] = vFacOrg[l + vPerm[i]];
			}
		}
	}
}
