﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace LogSig.DM.LsGen.Transformations
{
	public sealed class LsgtBlockSubst<T> : LsGenTransformation<T>
		where T : IDmGroupElement<T>
	{
		public override void Transform(TransformationParams<T> p)
		{
			var ls = p.LogSig;
			int n = ls.Blocks.Count;
			const int nBlocks = 2;

			for(int r = 0; r < n; ++r)
			{
				int x = p.Random.GenerateInt32(ls.Blocks.Count - nBlocks + 1);

				var lsSub = ls.GetSubSig(x, nBlocks);
				var lsNewSub = lsSub.GetRandomEquivalentCanonical(p.Group, p.Random);
				if(lsSub != lsNewSub)
				{
					if(p.Factorization != null)
					{
						List<int> vPre = p.Factorization.Skip(x).Take(nBlocks).ToList();
						T tProd = lsSub.MultiplyElements(vPre, p.Group);
						int[] vPost = new int[nBlocks];
						if(!DmLogSigUtil.TrivialFactorize(lsNewSub, tProd, p.Group.Identity,
							0, p.Group, vPost)) { Debug.Assert(false); }
						for(int iCpy = 0; iCpy < nBlocks; ++iCpy)
							p.Factorization[x + iCpy] = vPost[iCpy];
					}

					for(int xc = 0; xc < nBlocks; ++xc)
						ls.Blocks[x + xc] = lsNewSub.Blocks[xc];
				}
			}
		}
	}
}
