﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Numerics;
using System.Diagnostics;

using LogSig.Crypto;
using LogSig.Util;

namespace LogSig.DM.MultiZnGroup
{
	public sealed partial class DmMultiZnGroup : IDmGroup<DmMultiZnGroupElement>
	{
		/* public static DmBlockSeq<DmMultiZnGroupElement> GenNonRedei(int nBlocks)
		{
			if(nBlocks < 3) throw new ArgumentOutOfRangeException("nBlocks");

			int nCmps = 15 + (4 * (nBlocks - 2));
			BigInteger[] vMods = new BigInteger[nCmps];
			for(int i = 0; i < vMods.Length; ++i) vMods[i] = 2;

			DmMultiZnGroup g = new DmMultiZnGroup(vMods);

			var Bs = new List<DmMultiZnGroupElement>();
			Bs.Add(g.Identity);
			for(int i = 1; i <= nCmps; ++i)
			{
				BigInteger[] vEl = new BigInteger[nCmps];
				vEl[i - 1] = 1;
				Bs.Add(new DmMultiZnGroupElement(vEl));
			}

			var A1 = new DmLogSigBlock<DmMultiZnGroupElement>();
			A1.Elements.AddRange(new DmMultiZnGroupElement[] {
				g.Identity, Bs[1], Bs[2], Bs[3], Bs[1] + Bs[2],
				Bs[1] + Bs[3], Bs[2] + Bs[3] + Bs[4],
				Bs[1] + Bs[2] + Bs[3] + Bs[5]
			});

			var ls = new DmBlockSeq<DmMultiZnGroupElement>();

			return ls;
		} */

		public static void GenAperiodic(int p, int n, out DmMultiZnGroup grpOut,
			out DmBlockSeq<DmMultiZnGroupElement> lsOut)
		{
			if(p < 3) throw new ArgumentOutOfRangeException("p");
			if(n < 4) throw new ArgumentOutOfRangeException("n");

			List<BigInteger> lMods = new List<BigInteger>();
			for(int j = 0; j < 4; ++j) lMods.Add(new BigInteger(p));

			DmMultiZnGroup grp = new DmMultiZnGroup(lMods.ToArray());
			NormalChainEq eq = new NormalChainEq(grp, new DmBlockSeq<DmMultiZnGroupElement>());

			// List<DmMultiZnGroupElement> l = new List<DmMultiZnGroupElement>();
			// for(int i = 0; i < (p * p); ++i) l.Add(grp.Identity);
			DmBlockSeq<DmMultiZnGroupElement> ls = new DmBlockSeq<DmMultiZnGroupElement>();
			// ls.AddBlock(l.ToArray());
			// ls.AddBlock(l.ToArray());

			/* while(true)
			{
				// DmLogSigUtil.RandomizeCanonicalEnsureGenerating(ls, grp,
				//	Program.CryptoRandom);
				DmLogSigUtil.RandomizeCanonical(ls, grp, Program.CryptoRandom);
				if(!DmLogSigUtil.GeneratesWholeGroup(ls, grp)) continue;

				if(IsSubGroupBlock(ls.Blocks[0].Elements, grp, eq)) continue;
				if(IsSubGroupBlock(ls.Blocks[1].Elements, grp, eq)) continue;

				List<DmMultiZnGroupElement> lBlkB = new List<DmMultiZnGroupElement>();
				List<DmMultiZnGroupElement> lBlkS = new List<DmMultiZnGroupElement>();
				SplitIfPeriodicBlock(ls.Blocks[0].Elements, grp, eq, lBlkB, lBlkS);
				if(((lBlkB.Count * lBlkS.Count) == (p * p * p * p)) &&
					(lBlkB.Count > 1) && (lBlkS.Count > 1)) continue;

				SplitIfPeriodicBlock(ls.Blocks[1].Elements, grp, eq, lBlkB, lBlkS);
				if(((lBlkB.Count * lBlkS.Count) == (p * p * p * p)) &&
					(lBlkB.Count > 1) && (lBlkS.Count > 1)) continue;

				break;
			} */

			DmMultiZnGroupElement ex = new DmMultiZnGroupElement(new BigInteger[] {
				BigInteger.One, BigInteger.Zero, BigInteger.Zero, BigInteger.Zero });
			DmMultiZnGroupElement ey = new DmMultiZnGroupElement(new BigInteger[] {
				BigInteger.Zero, BigInteger.One, BigInteger.Zero, BigInteger.Zero });
			DmMultiZnGroupElement ez = new DmMultiZnGroupElement(new BigInteger[] {
				BigInteger.Zero, BigInteger.Zero, BigInteger.One, BigInteger.Zero });
			DmMultiZnGroupElement eu = new DmMultiZnGroupElement(new BigInteger[] {
				BigInteger.Zero, BigInteger.Zero, BigInteger.Zero, BigInteger.One });
			List<DmMultiZnGroupElement> A1 = new List<DmMultiZnGroupElement>();
			List<DmMultiZnGroupElement> A2 = new List<DmMultiZnGroupElement>();
			for(int s = 0; s < p; ++s)
			{
				for(int i = 0; i < p; ++i)
				{
					DmMultiZnGroupElement ki = grp.Pow(ex, i);
					DmMultiZnGroupElement kg = ez;
					if(i >= 1) kg = grp.Multiply(kg, eu);
					DmLogSigBlock<DmMultiZnGroupElement> blkSpan =
						GetCyclicSpan(grp, eq, kg);
					foreach(var se in blkSpan.Elements)
						A1.Add(grp.Multiply(ki, se));

					DmMultiZnGroupElement li = grp.Pow(ey, i);
					DmMultiZnGroupElement lg = eu;
					if(i >= 1) lg = grp.Multiply(lg, grp.Multiply(ez, ez));
					blkSpan = GetCyclicSpan(grp, eq, lg);
					foreach(var se in blkSpan.Elements)
						A2.Add(grp.Multiply(li, se));
				}
			}
			ls.AddBlock(A1.Distinct(eq).ToArray());
			ls.AddBlock(A2.Distinct(eq).ToArray());
			Debug.Assert(DmLogSigUtil.GeneratesWholeGroup(ls, grp));

			for(int r = 5; r <= n; ++r)
			{
				int yNewCmp = Program.CryptoRandom.GenerateInt32(lMods.Count + 1);

				if(yNewCmp < lMods.Count)
				{
					lMods[yNewCmp] *= p;

					for(int x = 0; x < ls.Blocks.Count; ++x)
					{
						List<DmMultiZnGroupElement> lBlk = ls.Blocks[x].Elements;
						for(int y = 0; y < lBlk.Count; ++y)
							lBlk[y].Values[yNewCmp] *= p;
					}
				}
				else
				{
					lMods.Add(new BigInteger(p));

					for(int x = 0; x < ls.Blocks.Count; ++x)
					{
						List<DmMultiZnGroupElement> lBlk = ls.Blocks[x].Elements;
						for(int y = 0; y < lBlk.Count; ++y)
						{
							List<BigInteger> lVal = new List<BigInteger>(lBlk[y].Values);
							lVal.Add(BigInteger.Zero);
							lBlk[y] = new DmMultiZnGroupElement(lVal.ToArray());
						}
					}
				}

				grp = new DmMultiZnGroup(lMods.ToArray());
				eq = new NormalChainEq(grp, new DmBlockSeq<DmMultiZnGroupElement>());

				DmMultiZnGroupElement b = grp.GenerateRandomElement(Program.CryptoRandom);
				b.Values[yNewCmp] = (((b.Values[yNewCmp] * p) +
					(Program.CryptoRandom.GenerateInt32(p - 1) + 1)) % lMods[yNewCmp]);

				List<DmMultiZnGroupElement> B = new List<DmMultiZnGroupElement>();
				for(int pow = 0; pow < p; ++pow)
					B.Add(grp.Pow(b, pow));

				while(IsSubGroupBlock(B, grp, eq))
				{
					DmMultiZnGroupElement re = grp.GenerateRandomElement(Program.CryptoRandom);
					re.Values[yNewCmp] = ((re.Values[yNewCmp] * p) % lMods[yNewCmp]);
					B[B.Count - 1] = grp.Multiply(B[B.Count - 1], re);
				}

				List<int> I = ls.Blocks.Select((blk, i) => ((blk.Elements.Count <
					(p * p)) ? i : -1)).Where(j => (j >= 0)).ToList();

				float fEta = 2.0f / 3.0f;
				float f = (float)Program.CryptoRandom.GenerateInt32(int.MaxValue) /
					(float)int.MaxValue;
				if((I.Count == 0) || (f < fEta))
					ls.AddBlock(B.ToArray());
				else
				{
					int x = I[Program.CryptoRandom.GenerateInt32(I.Count)];

					var blkNew = new DmLogSigBlock<DmMultiZnGroupElement>();
					for(int i1 = 0; i1 < ls.Blocks[x].Elements.Count; ++i1)
					{
						for(int i2 = 0; i2 < B.Count; ++i2)
						{
							blkNew.Elements.Add(grp.Multiply(B[i2],
								ls.Blocks[x].Elements[i1]));
						}
					}
					ls.Blocks[x] = blkNew;
				}

				if(grp.Order <= 59049)
				{
					Debug.Assert(DmLogSigUtil.GeneratesWholeGroup(ls, grp));
				}
			}

			grpOut = grp;
			lsOut = ls;
		}
	}
}
