﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Diagnostics;

using LogSig.Util;

namespace LogSig.DM.WreathProductGroup
{
	public sealed class DmWreathProductGroupFactory : IDmGroupFactory
	{
		public object CreateInstance(string strGroup)
		{
			List<string> lParts = StrUtil.SplitLevel0(strGroup,
				new char[] { 'w', 'W' });
			if(lParts.Count != 2) return null;

			object grpH = Program.GroupFactories.ConstructGroup(lParts[0]);
			object grpP = Program.GroupFactories.ConstructGroup(lParts[1]);
			if((grpH == null) || (grpP == null)) return null;

			Type tElemH = DmGroupFactoryPool.GetGroupElementType(grpH.GetType());
			Type tElemP = DmGroupFactoryPool.GetGroupElementType(grpP.GetType());

			Assembly asm = Assembly.GetExecutingAssembly();
			foreach(Type t in asm.GetTypes())
			{
				if(t.Name.StartsWith(@"DmWreathProductGroup`"))
				{
					Type tProductGroup = t.MakeGenericType(tElemH, tElemP);
					var vCtor = tProductGroup.GetConstructors();
					if((vCtor == null) || (vCtor.Length != 1))
						throw new InvalidOperationException();

					return vCtor[0].Invoke(new object[] { grpH, grpP });
				}
			}

			Debug.Assert(false);
			throw new InvalidOperationException();
		}
	}
}
