﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Text;
using System.Numerics;
using System.Diagnostics;

namespace LogSig.DM.ZnGroup
{
	public sealed class DmZnGroupElement : IDmGroupElement<DmZnGroupElement>
	{
		private int? m_hash = null;

		private readonly BigInteger m_v;
		public BigInteger Value
		{
			get { return m_v; }
			// set { m_v = value; } // Immutable
		}

		public DmZnGroupElement()
		{
			m_v = 0;
		}

		public DmZnGroupElement(BigInteger biValue)
		{
			m_v = biValue;
		}

		public DmZnGroupElement Clone()
		{
			return (DmZnGroupElement)MemberwiseClone();
		}

		public override string ToString()
		{
			return NumUtil.FactorsToString(m_v, true);
		}

		public static DmZnGroupElement Parse(string str)
		{
			return new DmZnGroupElement(NumUtil.Parse(str));
		}

		public override int GetHashCode()
		{
			if(m_hash.HasValue) return m_hash.Value;

			int h = m_v.GetHashCode();

			m_hash = h;
			return h;
		}

		public override bool Equals(object obj)
		{
			DmZnGroupElement other = (obj as DmZnGroupElement);
			return Equals(other);
		}

		public bool Equals(DmZnGroupElement other)
		{
			if(other == null) return false;
			return (m_v == other.m_v);
		}

		public int CompareTo(DmZnGroupElement other)
		{
			if(other == null) return 1;
			return m_v.CompareTo(other.m_v);
		}
	}
}
