﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Runtime.Remoting;
using System.IO;
using System.Diagnostics;

namespace LogSig.Plugins
{
	public static class LsPluginManager
	{
		private static DefaultPluginHost m_host = new DefaultPluginHost();
		private static List<LsPlugin> m_vPlugins = new List<LsPlugin>();

		public static void LoadAllPlugins()
		{
			string strExe = Assembly.GetExecutingAssembly().Location;
			FileInfo fiExe = new FileInfo(strExe);
			DirectoryInfo di = fiExe.Directory;
			FileInfo[] vPlugins = di.GetFiles("*.dll", SearchOption.AllDirectories);

			foreach(FileInfo fiPlugin in vPlugins)
			{
				try
				{
					FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(
						fiPlugin.FullName);
					if(!fvi.ProductName.Equals("LogSig Plugin")) continue;

					LsPlugin plg = CreatePluginInstance(fiPlugin.FullName);
					if(plg != null)
					{
						if(plg.Initialize(m_host)) m_vPlugins.Add(plg);
					}
				}
				catch(Exception)
				{
					Console.WriteLine("Failed to load plugin:");
					Console.WriteLine(fiPlugin.FullName);
				}
			}
		}

		private static LsPlugin CreatePluginInstance(string strFilePath)
		{
			if(strFilePath == null) throw new ArgumentNullException("strFilePath");

			string strShort = Path.GetFileNameWithoutExtension(strFilePath);
			string strType = strShort + "." + strShort + "Ext";
			ObjectHandle oh = Activator.CreateInstanceFrom(strFilePath, strType);

			return (oh.Unwrap() as LsPlugin);
		}

		public static void TerminateAllPlugins()
		{
			foreach(LsPlugin plg in m_vPlugins) { plg.Terminate(); }

			m_vPlugins.Clear();
		}
	}
}
