﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Numerics;
using System.Diagnostics;

using LogSig.DM;
using LogSig.DM.SnGroup;

namespace LogSig.Test
{
	public static class DmSnGroupTests
	{
		public static void Run()
		{
			TestZMapAndDefFact();
		}

		private static void TestZMapAndDefFact()
		{
			DmSnGroup grp = new DmSnGroup(3628800); // 10!

			var ls = grp.CreateDefaultSig();
			if(ls.Blocks.Count != 9) throw new Exception("DefSig");

			for(int i = 0; i < 100; ++i)
			{
				BigInteger bi = Program.CryptoRandom.GenerateBigInt(grp.Order);
				DmSnGroupElement el = grp.BigIntToElement(bi);
				BigInteger bi2 = grp.ElementToBigInt(el);

				if(bi != bi2) { Debug.Assert(false); throw new Exception("ZMap"); }

				List<int> lFact = new List<int>(grp.FactorInDefaultSig(el));
				DmSnGroupElement elP = ls.MultiplyElements(lFact, grp);
				if(elP.CompareTo(el) != 0) throw new Exception("DefFact");
			}
		}
	}
}
