﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;

using LogSig.DM;

namespace LogSig.Test
{
	public static class NumUtilTests
	{
		public static void Run()
		{
			TestFactorization();
			TestPerm();
			TestMath();
		}

		private static void TestFactorization()
		{
			if(NumUtil.GetPow(5 * 7 * 11 * 13, 2) != 0)
				throw new Exception("Get2Pow(5 * 7 * 11 * 13) != 0");
			if(NumUtil.GetPow(2 * 3 * 5, 2) != 1)
				throw new Exception("Get2Pow(2 * 3 * 5) != 1");
			if(NumUtil.GetPow(2 * 2 * 2 * 2 * 2 * 3 * 7 * 11, 2) != 5)
				throw new Exception("Get2Pow(2 * 2 * 2 * 2 * 2 * 3 * 7 * 11) != 5");

			if(NumUtil.CountOneBits(4256) != 3)
				throw new Exception("CountOneBits(4256) != 3");
			if(NumUtil.CountOneBits(35147) != 7)
				throw new Exception("CountOneBits(35147) != 7");

			List<int> v = NumUtil.GetOneBitIndices(357);
			if(v.Count != 5) throw new Exception("v.Count != 5");
			if(v[0] != 0) throw new Exception("v[0] != 0");
			if(v[1] != 2) throw new Exception("v[1] != 2");
			if(v[2] != 5) throw new Exception("v[2] != 5");
			if(v[3] != 6) throw new Exception("v[3] != 6");
			if(v[4] != 8) throw new Exception("v[4] != 8");

			List<BigInteger> l = NumUtil.Factorize(2 * 3 * 5 * 7);
			if(l.Count != 4) throw new Exception("l.Count != 4");

			string str = NumUtil.FactorsToString(l);
			if(str != "2*3*5*7") throw new Exception("str != 2*3*5*7");

			l = NumUtil.Factorize(2 * 2 * 2 * 3 * 5 * 5 * 7 * 7 * 7);
			if(l.Count != 9) throw new Exception("l.Count != 9");

			str = NumUtil.FactorsToString(l);
			if(str != "2^3*3*5^2*7^3") throw new Exception("str != 2^3*3*5^2*7^3");

			l = NumUtil.Factorize(7 * 11 * 13 * 13 * 13 * 17);
			if(l.Count != 6) throw new Exception("l.Count != 6");

			str = NumUtil.FactorsToString(l);
			if(str != "7*11*13^3*17") throw new Exception("str != 7*11*13^3*17");
		}

		private static void TestPerm()
		{
			int[] v = new int[5];
			for(int a = 0; a < v.Length; ++a) v[a] = a;

			for(int i = 0; i < 120; ++i)
			{
				if((i == 9) && ((v[0] != 0) || (v[1] != 2) ||
					(v[2] != 3) || (v[3] != 4) || (v[4] != 1)))
					throw new Exception("Perm-9");
				if((i == 19) && ((v[0] != 0) || (v[1] != 4) ||
					(v[2] != 1) || (v[3] != 3) || (v[4] != 2)))
					throw new Exception("Perm-19");
				if((i == 79) && ((v[0] != 3) || (v[1] != 1) ||
					(v[2] != 0) || (v[3] != 4) || (v[4] != 2)))
					throw new Exception("Perm-79");
				if((i == 99) && ((v[0] != 4) || (v[1] != 0) ||
					(v[2] != 2) || (v[3] != 3) || (v[4] != 1)))
					throw new Exception("Perm-99");
				if((i == 119) && ((v[0] != 4) || (v[1] != 3) ||
					(v[2] != 2) || (v[3] != 1) || (v[4] != 0)))
					throw new Exception("Perm-119");

				NumUtil.NextPermutation(v);
			}

			if((v[0] != 0) || (v[1] != 1) || (v[2] != 2) ||
				(v[3] != 3) || (v[4] != 4))
				throw new Exception("Perm-120");
		}

		private static void TestMath()
		{
			BigInteger n = NumUtil.Factorial(6);
			if(n != 720) throw new Exception("6! != 720");
		}
	}
}
