﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Numerics;

namespace LogSig.Util
{
	public sealed class BitStream
	{
		private Stream m_s;

		private bool m_bEos = false;
		private byte m_btBufBits;
		private int m_nBufBits = 0;

		public BitStream(Stream s)
		{
			if(s == null) throw new ArgumentNullException("s");

			m_s = s;
		}

		public bool? ReadBit()
		{
			if(m_bEos) return null;

			if(m_nBufBits == 0)
			{
				int bt = m_s.ReadByte();
				if(bt < 0) { m_bEos = true; return null; }

				m_btBufBits = (byte)bt;
				m_nBufBits = 8;
			}

			bool bBit = ((m_btBufBits & 1) != 0);
			m_btBufBits >>= 1;
			--m_nBufBits;
			return bBit;
		}

		public BigInteger? ReadBitsAsBigInt(int nMaxBits)
		{
			if(m_bEos) return null;

			BigInteger bi = BigInteger.Zero;
			if(nMaxBits <= 0) return bi;

			bool bAtLeastOneBit = false;
			for(int i = 0; i < nMaxBits; ++i)
			{
				bool? bBit = ReadBit();
				if(!bBit.HasValue) break;

				bi <<= 1;
				if(bBit.Value) bi |= 1;
			}

			if(!bAtLeastOneBit) return null;
			return bi;
		}
	}
}
