﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using System.Diagnostics;

using LogSig.Interfaces;

namespace LogSig.Util
{
	public sealed class LpFileGenerator : IIlpSolver
	{
		private int m_nVars;
		private Type[] m_vTypes;
		private List<Tuple<double[], ConstraintType, double>> m_lConstr;

		public LpFileGenerator(int nVars)
		{
			m_nVars = nVars;
			m_lConstr = new List<Tuple<double[], ConstraintType, double>>();
			m_vTypes = new Type[m_nVars];
		}

		public bool AddConstraint(double[] vRow, ConstraintType t, double rh)
		{
			double[] v = (double[])vRow.Clone();
			m_lConstr.Add(new Tuple<double[], ConstraintType, double>(v, t, rh));
			return true;
		}

		public bool SetTypeBinary(int iCol)
		{
			m_vTypes[iCol] = typeof(bool);
			return true;
		}

		public bool SetTypeInt(int iCol)
		{
			m_vTypes[iCol] = typeof(int);
			return true;
		}

		public double[] Solve()
		{
			throw new InvalidOperationException();
		}

		public int[] SolveInt()
		{
			int[] v = new int[m_nVars];
			return v;
		}

		public string BuildLP()
		{
			StringBuilder sb = new StringBuilder();
			NumberFormatInfo nfi = NumberFormatInfo.InvariantInfo;

			sb.AppendLine("Subject To");
			for(int ic = 0; ic < m_lConstr.Count; ++ic)
			{
				var c = m_lConstr[ic];

				sb.Append(" C");
				sb.Append(ic);
				sb.Append(": ");

				bool bFirstVar = true;
				for(int x = 0; x < c.Item1.Length; ++x)
				{
					double f = c.Item1[x];
					if(f == 0.0) continue;

					if(bFirstVar)
					{
						if(f < 0.0) sb.Append(" - ");
					}
					else sb.Append((f >= 0.0) ? " + " : " - ");

					if(Math.Abs(f) != 1.0)
					{
						sb.Append(Math.Abs(f).ToString(nfi));
						sb.Append(' ');
					}

					sb.Append("x");
					sb.Append(x);

					bFirstVar = false;
				}

				switch(c.Item2)
				{
					case ConstraintType.EQ: sb.Append(" = "); break;
					case ConstraintType.LE: sb.Append(" <= "); break;
					case ConstraintType.GE: sb.Append(" >= "); break;
					default: throw new InvalidOperationException();
				}

				sb.AppendLine(c.Item3.ToString(nfi));
			}

			if(m_vTypes.Contains(typeof(int)))
			{
				sb.AppendLine("Bounds");
				for(int x = 0; x < m_nVars; ++x)
				{
					if(m_vTypes[x] == typeof(int))
						sb.AppendLine(" x" + x.ToString() + " >= 0");
				}
			}

			if(m_vTypes.Contains(typeof(int)))
			{
				sb.AppendLine("Integers");
				for(int x = 0; x < m_nVars; ++x)
				{
					if(m_vTypes[x] == typeof(int))
						sb.Append(" x" + x.ToString());
				}
				sb.AppendLine();
			}
			if(m_vTypes.Contains(typeof(bool)))
			{
				sb.AppendLine("Binaries");
				for(int x = 0; x < m_nVars; ++x)
				{
					if(m_vTypes[x] == typeof(bool))
						sb.Append(" x" + x.ToString());
				}
				sb.AppendLine();
			}

			sb.AppendLine("End");
			return sb.ToString();
		}
	}
}
