﻿// LogSig - Logarithmic Signature Utility
// Copyright (C) 2011-2015 Dominik Reichl <info@dominik-reichl.de>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace LogSig.Util
{
	public static class StrUtil
	{
		public const StringComparison CaseIgnoreCmp = StringComparison.OrdinalIgnoreCase;

		public static string BlocksOnNewLines(string str, int nBreakLevel)
		{
			StringBuilder sb = new StringBuilder();
			int lvl = 0;

			for(int i = 0; i < str.Length; ++i)
			{
				char ch = str[i];

				if(ch == '(') ++lvl;
				else if(ch == ')') --lvl;

				if((ch == ' ') && (lvl == nBreakLevel))
					sb.Append(Environment.NewLine);
				else sb.Append(ch);
			}

			return sb.ToString();
		}

		public static string ToString(Array ar)
		{
			StringBuilder sb = new StringBuilder();
			sb.Append("(");

			for(int i = 0; i < ar.Length; ++i)
			{
				if(i > 0) sb.Append(", ");
				sb.Append(ar.GetValue(i).ToString());
			}

			sb.Append(")");
			return sb.ToString();
		}

		public static string TrimEx(string str)
		{
			str = str.Trim();

			StringBuilder sb = new StringBuilder();
			bool bLastWhite = false;

			foreach(char ch in str)
			{
				bool bWhite = char.IsWhiteSpace(ch);

				if(!bWhite)
				{
					sb.Append(ch);
					bLastWhite = false;
				}
				else if(bWhite && !bLastWhite)
				{
					sb.Append(' ');
					bLastWhite = true;
				}
			}

			return sb.ToString();
		}

		public static List<string> SplitLevel0(string str, char[] vSplitChars)
		{
			List<string> l = new List<string>();
			StringBuilder sb = new StringBuilder();
			int iLevel = 0;

			foreach(char ch in str)
			{
				if(ch == '(') ++iLevel;
				else if(ch == ')') --iLevel;

				if(iLevel >= 1) sb.Append(ch);
				else
				{
					if(Array.IndexOf<char>(vSplitChars, ch) >= 0)
					{
						l.Add(sb.ToString());
						sb.Clear();
					}
					else sb.Append(ch);
				}
			}

			l.Add(sb.ToString());
			return l;
		}
	}
}
